#!/usr/bin/env node

const {exec, execSync} = require('child_process')
const path = require('path')
const readline = require('readline')
const fs = require('fs')
const os = require('os')
const { Logger } = require('../../utils/Logger')
const osType = os.type()
async function getSourceBranch(){
    return new Promise((resolve,reject) => {
        const sourceBranch = readline.createInterface({
            input: process.stdin,
            output: process.stdout
        })
        sourceBranch.question('\x1b[33mEnter current working branch: \x1b[0m',branchName => {
            resolve(branchName)
            sourceBranch.close()
        })
    })
}
async function getTargetBranch(){
    return new Promise((resolve, reject) => {
        const targetBranch = readline.createInterface({
            input: process.stdin,
            output: process.stdout
        })
        targetBranch.question('\x1b[33mEnter the branch to be compared with: \x1b[0m',branchName => {
            resolve(branchName)
            targetBranch.close()
        }) 
    })
}
async function getChangedFilesNames(sourceBranch,targetBranch){
    return new Promise((resolve,reject) => {
        exec(`git diff ${targetBranch}...${sourceBranch} --name-only`,(error,stderr,stdout) => {
            if(error){
                reject("Couldn't get the diff between branches",error)
            }
            else if(stderr){
                resolve(stderr.trim().split('\n'))
            }
            else if(stdout){
                resolve(stdout.trim().split('\n'))
            }
        })
    })
}
function executeGitFetch(){
    let gitFetchOutPut = ''
    try{
        gitFetchOutPut = execSync(`git fetch 2>&1`).toString()
        if(gitFetchOutPut.includes('From https://zgit.csez.zohocorpin.com/zohodesk/desk_client_app')){
            return true
        }
        else if(gitFetchOutPut == ''){
            return true
        }
        else if(gitFetchOutPut == undefined){
            return true
        }
    }
    catch(error){
        if(error.stdout.toString().includes(`fatal: unable to access 'https://zgit.csez.zohocorpin.com/zohodesk/desk_client_app.git/'`)){
            return false
        }
        else if(error.stdout.toString().includes('error: cannot lock ref')){
            return true
        }
        else if(error.stdout.toString() == ''){
            return true
        }
        else if(error == undefined){
            return true
        }
        else{
            return true
        }
    }
}
async function findEslintErrors(){
    return new Promise((resolve,reject) => {
        try{
            exec(`npx eslint --no-eslintrc -c ./eslint/conf/.eslintrc.js --no-inline-config --resolve-plugins-relative-to ='./node_modules' temp.js`,(error,stderr,stdout) => {
                if(stderr){
                    resolve(stderr.trim().split('\n'))
                }
                else if(error){
                    console.log(error)
                    reject("Error encountered while executing eslint command")
                }
                else{
                    resolve([])
                }
            })
        }
        catch(err){
            Logger.log(Logger.FAILURE_TYPE,"Error in executing the eslint command");
        }
    })
}
function isFilePresentInTargetBranch(targetBranch,fileName){
    try{
        let gitShowOutput = execSync(`git show ${targetBranch}:${fileName} 2>&1`).toString().trim()
        if(gitShowOutput.includes('fatal: Path')){
            return false
        }
        else{
            return true
        }
    }
    catch(error){
    }
}
function isFilePresentInTargetBranchWindows(targetBranch,fileName){
    try{
        let gitShowOutput = execSync(`git show ${targetBranch}:${fileName} 2> NUL`).toString()
        if(gitShowOutput.includes('fatal: Path')){
            return false
        }
        else{
            return true
        }
    }
    catch(error){
    }
}
function getLastLineNumberOfFile(fileName){
    try{
        let fileContent  = fs.readFileSync(fileName,'utf8')
        let fileContentArray = fileContent.split('\n')

        return fileContentArray.length
    }
    catch(error){
        Logger.log(Logger.FAILURE_TYPE,"Error in getting the last line number of file")
    }
}
async function findChangedLinesBetweenBranches(currentWorkingBranch,branchToBeComparedWith,fileName){
    return new Promise((resolve,reject) => {
        exec(`git diff -U0 ${branchToBeComparedWith}:${fileName} ${currentWorkingBranch}:${fileName}`,(error,stderr,stdout) => {
            if(stderr){
                resolve(stderr.trim().split('\n'))
            }
            else if(error){
                reject(error)
            }
            else if(stdout){
                resolve(stdout.trim().split('\n'))
            }
        })
    })    

}
async function showEslintErrorsInDiffBetweenBranches(){

    //executing git fetch
    Logger.log(Logger.INFO_TYPE,osType)
    Logger.log(Logger.INFO_TYPE,"git fetch in progress...")
    if(executeGitFetch()){
        Logger.log(Logger.INFO_TYPE,'Git fetch successful!')
        try{
            //getting source and target branches as input from user
            let currentWorkingBranch = 'origin/' + await getSourceBranch()
            let branchToBeComparedWith = 'origin/' + await getTargetBranch()
    
            //getting the files that are changed
            let changedFiles = await getChangedFilesNames(currentWorkingBranch,branchToBeComparedWith)
            for (let file of changedFiles){
                if(isFilePresentInTargetBranch(currentWorkingBranch,file)){
                    if(file.includes(staged_files[file] && typeof(staged_files[file]) == 'string')){
                        if(path.extname(file) === '.js'){
                            execSync(`git show ${currentWorkingBranch}:${file} > temp.js`)
                            try{
                                let changedLinesStartArray = []
                                let changedLinesEndArray = []
                                if(isFilePresentInTargetBranch(branchToBeComparedWith,file)){
                                    let changesLineNumbers = []
                                    let changedLines = await findChangedLinesBetweenBranches(currentWorkingBranch,branchToBeComparedWith,file)
                                    changesLineNumbers = changedLines.filter(line => line.startsWith('@@'))
                                    for(let lineNumber of changesLineNumbers){
                                        let changesStartLine = parseInt(lineNumber.trim().split(' ')[2].split(',')[0])
                                        changedLinesStartArray.push(changesStartLine)
                                        let changesEndLine = lineNumber.trim().split(' ')[2].split(',')[1]
                                        if(changesEndLine === undefined){
                                            changedLinesEndArray.push(changesStartLine)
                                        }
                                        else{
                                            changedLinesEndArray.push(changesStartLine + parseInt(changesEndLine) -1)
                                        }
                                    }
                                }
                                else{
                                    let lineNumberOfLastLine = getLastLineNumberOfFile(`temp.js`)
                                    changedLinesStartArray.push(1)
                                    changedLinesEndArray.push(lineNumberOfLastLine)
                                }
                                let eslintErrorsInFile = await findEslintErrors()
                                if(! eslintErrorsInFile.length == 0){
                                    let eslintErrorLineNumbers = []
                                    let eslintErrorsInChangedLines = []
                                    for(let errorLine of eslintErrorsInFile){
                                        if(/^\d/.test(errorLine.trim())){
                                            let errorLineNumber = parseInt(errorLine.trim().split(' ')[0].split(':')[0])
                                            eslintErrorLineNumbers.push(errorLineNumber)
                                        }
                                    }
                                    for(let eslintError = 0; eslintError < eslintErrorLineNumbers.length; eslintError++){
                                        for(let changedLine=0; changedLine < changedLinesStartArray.length; changedLine++){
                                            if(eslintErrorLineNumbers[eslintError] >= changedLinesStartArray[changedLine] &&
                                                eslintErrorLineNumbers[eslintError] <= changedLinesEndArray[changedLine]){
                                                    eslintErrorsInChangedLines.push(eslintErrorsInFile[eslintError + 1])
                                                }
                                        }
                                    }
                                    if(eslintErrorsInChangedLines.length > 0){
                                        Logger.log(Logger.INFO_TYPE,file)
                                        for(let eslintErrorInChangedLine of eslintErrorsInChangedLines){
                                            Logger.log(Logger.INFO_TYPE,eslintErrorInChangedLine)
                                        }
                                        try{
                                            fs.unlinkSync('temp.js')
                                        }
                                        catch(err){
                                            Logger.log(Logger.INFO_TYPE,'Unable to remove file(Error occurred in if statement)',file,err)
                                        } 
                                    } 
                                    else{
                                        Logger.log(Logger.INFO_TYPE,file)
                                        try{
                                            fs.unlinkSync('temp.js')
                                        }
                                        catch(err){
                                            Logger.log(Logger.INFO_TYPE,'Unable to remove file(Error occurred in else statement)',file,err)
                                        } 
                                    }      
                                }
                                else{
                                    Logger.log(Logger.INFO_TYPE,file)
                                    try{
                                        fs.unlinkSync('temp.js')
                                    }
                                    catch(err){
                                        Logger.log(Logger.INFO_TYPE,`Unable to remove file(Error occurred in else block of condition where no eslint error in file${file,err}`)
                                    } 
                                }
                                
                            }
                            catch(error){
                                Logger.log(Logger.INFO_TYPE,"Some error occurred",error)
                            }    
                        }
                    }
                }
            }
        }
        catch{
            Logger.log(Logger.INFO_TYPE,'Some issue with getting input from user')
        }
    }
    else{
        Logger.log(Logger.INFO_TYPE,'Error in executing git fetch')
        Logger.log(Logger.INFO_TYPE,"Make sure https://zgit.csez.zohocorpin.com/zohodesk/desk_client_app.git is accessible")
    }
}

//windows
async function showEslintErrorsInDiffBetweenBranchesWindows(){

    //executing git fetch
    Logger.log(Logger.INFO_TYPE,"git fetch in progress...")
    if(executeGitFetch()){
        Logger.log(Logger.INFO_TYPE,'Git fetch successful!')
        try{
            //getting source and target branches as input from user
            let currentWorkingBranch = 'origin/' + await getSourceBranch()
            let branchToBeComparedWith = 'origin/' + await getTargetBranch()
    
            //getting the files that are changed
            let changedFiles = await getChangedFilesNames(currentWorkingBranch,branchToBeComparedWith)
            for (let file of changedFiles){
                if(isFilePresentInTargetBranchWindows(currentWorkingBranch,file)){
                    if(file.includes('jsapps/supportapp/src') && !file.includes('.docs.js') && !file.includes('.spec.js') && !file.includes('.specs.js') && !file.includes('.test.js')){
                        if(path.extname(file) === '.js'){
                            let gitShowContent = execSync(`git show ${currentWorkingBranch}:${file}`)
                            fs.writeFileSync('temp.js',gitShowContent)
                            try{
                                let changedLinesStartArray = []
                                let changedLinesEndArray = []
                                if(isFilePresentInTargetBranchWindows(branchToBeComparedWith,file)){
                                    let changesLineNumbers = []
                                    let changedLines = await findChangedLinesBetweenBranches(currentWorkingBranch,branchToBeComparedWith,file)
                                    changesLineNumbers = changedLines.filter(line => line.startsWith('@@'))
                                    for(let lineNumber of changesLineNumbers){
                                        let changesStartLine = parseInt(lineNumber.trim().split(' ')[2].split(',')[0])
                                        changedLinesStartArray.push(changesStartLine)
                                        let changesEndLine = lineNumber.trim().split(' ')[2].split(',')[1]
                                        if(changesEndLine === undefined){
                                            changedLinesEndArray.push(changesStartLine)
                                        }
                                        else{
                                            changedLinesEndArray.push(changesStartLine + parseInt(changesEndLine) -1)
                                        }
                                    }
                                }
                                else{
                                    let lineNumberOfLastLine = getLastLineNumberOfFile(`temp.js`)
                                    changedLinesStartArray.push(1)
                                    changedLinesEndArray.push(lineNumberOfLastLine)
                                }
                                let eslintErrorsInFile = await findEslintErrors()
                                if(! eslintErrorsInFile.length == 0){
                                    let eslintErrorLineNumbers = []
                                    let eslintErrorsInChangedLines = []
                                    for(let errorLine of eslintErrorsInFile){
                                        if(/^\d/.test(errorLine.trim())){
                                            let errorLineNumber = parseInt(errorLine.trim().split(' ')[0].split(':')[0])
                                            eslintErrorLineNumbers.push(errorLineNumber)
                                        }
                                    }
                                    for(let eslintError = 0; eslintError < eslintErrorLineNumbers.length; eslintError++){
                                        for(let changedLine=0; changedLine < changedLinesStartArray.length; changedLine++){
                                            if(eslintErrorLineNumbers[eslintError] >= changedLinesStartArray[changedLine] &&
                                                eslintErrorLineNumbers[eslintError] <= changedLinesEndArray[changedLine]){
                                                    eslintErrorsInChangedLines.push(eslintErrorsInFile[eslintError + 1])
                                                }
                                        }
                                    }
                                    if(eslintErrorsInChangedLines.length > 0){
                                        Logger.log(Logger.INFO_TYPE,file)
                                        for(let eslintErrorInChangedLine of eslintErrorsInChangedLines){
                                            Logger.log(Logger.INFO_TYPE,eslintErrorInChangedLine)
                                        }
                                        try{
                                            fs.unlinkSync('temp.js')
                                        }
                                        catch(err){
                                            Logger.log(Logger.INFO_TYPE,'Unable to remove file(Error occurred in if statement in windows method)',file,err)
                                        } 
                                    } 
                                    else{
                                        Logger.log(Logger.INFO_TYPE,`\x1b[32m${file}\x1b[0m`)
                                        try{
                                            fs.unlinkSync('temp.js')
                                        }
                                        catch(err){
                                            Logger.log(Logger.INFO_TYPE,`Unable to remove file(Error occurred in else block in Windows method)${file,err}`)
                                        } 
                                    }      
                                }
                                else{
                                    Logger.log(Logger.INFO_TYPE,`\x1b[32m${file}\x1b[0m`)
                                    try{
                                        fs.unlinkSync('temp.js')
                                    }
                                    catch(err){
                                        Logger.log(Logger.INFO_TYPE,`Unable to remove file(Error occurred in else block in Windows method)${file,err}`)
                                    } 
                                }
                                
                            }
                            catch(error){
                                Logger.log(Logger.INFO_TYPE,"Some error occurred",error)
                            }    
                        }
                    }
                }
            }
        }
        catch{
            Logger.log(Logger.INFO_TYPE,'Some issue with getting input from user')
        }
    }
    else{
        Logger.log(Logger.INFO_TYPE,'Error in executing git fetch')
        Logger.log(Logger.INFO_TYPE,"Make sure https://zgit.csez.zohocorpin.com/zohodesk/desk_client_app.git is accessible")
    }
}
if(osType == 'Darwin' || osType == 'Linux'){
    showEslintErrorsInDiffBetweenBranches()
}
else if (osType == 'Windows_NT'){
    showEslintErrorsInDiffBetweenBranchesWindows()
}
