/* eslint-disable no-console */

const {execSync} = require('child_process')
const fs = require('fs')
const path = require('path')
const removeFile = require('../../../build/removeFile')
const grantExecutionPermission = require('../../../build/grantExecutionPermission')
const { getShellScript } = require('../../../utils/getScripts')
const { Logger } = require('../../../utils/Logger')

function configurePrecommitHook(){
    try{
        removeFile()
        execSync(`npx husky add .husky/pre-commit "${getShellScript()}"`)
        try{
            let currentWorkingDirectory = process.cwd()
            let huskyPrecommitPath = path.join(currentWorkingDirectory,'.husky/pre-commit')
            let huskyPrecommitHookContent = fs.readFileSync(huskyPrecommitPath,'utf-8').split('\n')
            huskyPrecommitHookContent = huskyPrecommitHookContent.map(line => {
                if(line.includes('@zohodesk/precommithook')){
                    return `"${line}"`
                }
                else{
                    return `${line}`
                }
            })
            try{
                fs.writeFileSync(huskyPrecommitPath,huskyPrecommitHookContent.join('\n'),'utf-8')
                grantExecutionPermission()
                return true
            }
            catch(error){
                Logger.log(Logger.FAILURE_TYPE,error)
                Logger.log(Logger.FAILURE_TYPE,'Error adding double quotes to husky precommit file')
                return false
            }
        }
        catch(error){
            Logger.log(Logger.FAILURE_TYPE,error)
            Logger.log(Logger.FAILURE_TYPE,'Couldn\'t read the contents of husky precommit hook file')
            return false
        }
    }
    catch(error){
        Logger.log(Logger.FAILURE_TYPE,`Couldn\'t link husky pre commit hook to custom pre commit hook ${error.toString()}\n`)
        return false
    }
}

module.exports = configurePrecommitHook